static bool verbose=true;
#define FTDI_BAUDRATE 3000000
#define FTDI_TIMEOUT_MS 1000
#ifdef FTD2XX_H_LINUX
//we use libusb to disconnect the ftdi_sio driver that tends to be loaded automatically on most systems
//in theory we could avoid this with udev rules, in practice coming up with a robust and portable solution is tricky...
#include <libusb-1.0/libusb.h>
static void libusb_disconnect_driver(void){
    struct libusb_context *usb_ctx;
    struct libusb_device_handle *usb_dev;
    libusb_device *dev;
    libusb_device **devs;
    int count = 0;
    int i = 0;

	if (libusb_init(&usb_ctx) < 0)
        throw new std::string("libusb_init() failed");

	try{
		if (libusb_get_device_list(usb_ctx, &devs) < 0)
			throw new std::string("libusb_get_device_list() failed");

		while ((dev = devs[i++]) != NULL){
			char description[1024];
			int desc_len=sizeof(description);
			struct libusb_device_descriptor desc;

			if (libusb_get_device_descriptor(dev, &desc) < 0)
				throw new std::string("libusb_get_device_descriptor() failed");

			if ((desc.idVendor == 0x403) &&(desc.idProduct == 0x6001 || 
											desc.idProduct == 0x6010 ||
											desc.idProduct == 0x6011 ||
											desc.idProduct == 0x6014 ||
											desc.idProduct == 0x6015)) {
				if(verbose) printf("device found: vendor=%X, product=%X\n",desc.idVendor, desc.idProduct);
				if (libusb_open(dev, &usb_dev) < 0) throw new std::string("libusb_open() failed");
				if (libusb_get_device_descriptor(dev, &desc) < 0) throw new std::string("libusb_get_device_descriptor() failed");

				if (libusb_get_string_descriptor_ascii(usb_dev, desc.iProduct, (unsigned char *)description, desc_len) < 0){
					libusb_close(usb_dev);
					throw new std::string("libusb_get_string_descriptor_ascii() failed");
				}
				
				if(strcmp(description,"kidekin_trng")==0){
					if(verbose) printf("%s found\n",description);
					libusb_detach_kernel_driver(usb_dev, 0);
					libusb_detach_kernel_driver(usb_dev, 1);
				}
				libusb_close(usb_dev);
			}
		}
		libusb_free_device_list(devs,1);
	} catch (std::string *s){
		libusb_exit(usb_ctx);
		throw s;
	}
	libusb_exit(usb_ctx);	
}
#endif
void kidekin_trng_set_verbose(bool verbose_value){
	verbose = verbose_value;
}
