
#include <cstdlib>
#include <iostream>
#include <cstring>
#include <string.h>
#include "stdio.h"
#include "ftdi.h"

#define BUF_USABLE_SIZE (sizeof(buf))

static struct ftdi_context *ftdi=0;
static struct ftdi_context *ftdi2=0;
static struct ftdi_device_list *devlist=0;

static uint8_t buf[1024];
static int readPos;

#define FTD2XX_H_LINUX
#include "kidekin_trng_common.h"

unsigned int kidekin_trng_find_next(unsigned int start_index){
	unsigned int numDevs;
	int ret, i;
    int retval = EXIT_SUCCESS;
	struct ftdi_device_list *devlist, *curdev;
    #ifdef FTD2XX_H_LINUX
		libusb_disconnect_driver();
	#endif
	if ((ftdi = ftdi_new()) == 0){
        fprintf(stderr, "ftdi_new failed\n");
        return EXIT_FAILURE;
    }
	
	if ((ret = ftdi_usb_find_all(ftdi, &devlist, 0, 0)) < 0){
        fprintf(stderr, "ftdi_usb_find_all failed: %d (%s)\n", ret, ftdi_get_error_string(ftdi));
        retval =  EXIT_FAILURE;
        goto do_deinit;
    }
	numDevs=ret;
	if(verbose) printf("ftdi_usb_find_all OK, number of devices connected is =%d\n",numDevs); 
	if(numDevs<=start_index) throw new std::string("ERROR: index bigger than total number of devices");
	curdev = devlist;
	for(unsigned int index=0;index<start_index;index++){curdev=curdev->next;}//skip devices lower than start index
	for(unsigned int index=start_index;index<numDevs;index++){
		char manufacturer[128], description[128];
    
		if(verbose) printf("Checking device: %d\n", index);
        if ((ret = ftdi_usb_get_strings(ftdi, curdev->dev, manufacturer, 128, description, 128, NULL, 0)) < 0){
			printf("ftdi_usb_get_strings failed: %d (%s)\n", ret, ftdi_get_error_string(ftdi));
            fprintf(stderr, "ftdi_usb_get_strings failed: %d (%s)\n", ret, ftdi_get_error_string(ftdi));
            retval = EXIT_FAILURE;
            goto done;
        }
        if(verbose) printf("Manufacturer: %s, Description: %s\n", manufacturer, description);
        if(strcmp("kidekin_trng",description)==0){
			ftdi_list_free(&devlist);devlist=0;
			ftdi_free(ftdi);ftdi=0;
			return index;
		}
		curdev = curdev->next;
	}
	done:
    ftdi_list_free(&devlist);devlist=0;
do_deinit:
    ftdi_free(ftdi);ftdi=0;

	return -1;
	
}

//return false if the device could not be openned
bool kidekin_trng_open(unsigned int device_index){
	readPos=-1;
	unsigned int numDevs;
	struct ftdi_device_list *curdev;
    char manufacturer[128], description[128];
    
	int ret, i;
    if(verbose) printf("kidekin_trng_open()\n");
	if ((ftdi = ftdi_new()) == 0){
        fprintf(stderr, "ftdi_new failed\n");
        return false;
    }
	
	if ((ret = ftdi_usb_find_all(ftdi, &devlist, 0, 0)) < 0){
        fprintf(stderr, "ftdi_usb_find_all failed: %d (%s)\n", ret, ftdi_get_error_string(ftdi));
        ftdi_list_free(&devlist);devlist=0;
		ftdi_free(ftdi);ftdi=0;
		return false;	
    }
	numDevs=ret;
	if(verbose) printf("ftdi_usb_find_all OK, number of devices connected is =%d\n",numDevs); 
	if(numDevs<=device_index) throw new std::string("ERROR: index bigger than total number of devices");
	curdev = devlist;
	for(unsigned int index=0;index<device_index;index++){curdev=curdev->next;}//skip devices lower than device index

	if(verbose) printf("Checking device: %d\n", device_index);
	if ((ret = ftdi_usb_get_strings(ftdi, curdev->dev, manufacturer, 128, description, 128, NULL, 0)) < 0)
	{
		fprintf(stderr, "ftdi_usb_get_strings failed: %d (%s)\n", ret, ftdi_get_error_string(ftdi));
		ftdi_list_free(&devlist);devlist=0;
		ftdi_free(ftdi);ftdi=0;
		return false;	
	}
	if(verbose) printf("Manufacturer: %s, Description: %s\n", manufacturer, description);
	if(strcmp("kidekin_trng",description)==0){
			int status;    
			
		if ((ftdi2 = ftdi_new()) == 0){
			fprintf(stderr, "ftdi_new failed\n");
			ftdi_list_free(&devlist);devlist=0;
			ftdi_free(ftdi);ftdi=0;
			return false;
		}
		ftdi_set_interface(ftdi2, INTERFACE_B);

		if ( ftdi_usb_open_dev(ftdi2,curdev->dev) < 0){
			fprintf(stderr,"Can't open ftdi device: %s\n",ftdi_get_error_string(ftdi2));
			ftdi_list_free(&devlist);devlist=0;
			ftdi_free(ftdi);ftdi=0;
			ftdi_free(ftdi2);ftdi2=0;
			return false;
		}
		if(verbose) printf("Device opened\n");
		
		if(status=ftdi_set_baudrate(ftdi2,FTDI_BAUDRATE)<0){
			fprintf(stderr, "ftdi_set_baudrate:%d\n",status);
			ftdi_list_free(&devlist);devlist=0;
			ftdi_free(ftdi);ftdi=0;
			ftdi_free(ftdi2);ftdi2=0;
			return false;
		}
		// Set RTS/CTS flow control 
		if(status=ftdi_setflowctrl(ftdi2,SIO_RTS_CTS_HS)<0){
			fprintf(stderr, "ftdi_setflowctrl failed:%d\n",status);
			ftdi_list_free(&devlist);devlist=0;
			ftdi_free(ftdi);ftdi=0;
			ftdi_free(ftdi2);ftdi2=0;
			return false;
		}
		if(status=ftdi_set_line_property (ftdi2,
				BITS_8,     //enum ftdi_bits_type  	bits,
				STOP_BIT_1, //enum ftdi_stopbits_type  	sbit,
				NONE       //enum ftdi_parity_type  	parity
			)<0){
			fprintf(stderr, "ftdi_set_line_property failed:%d\n",status);
			ftdi_list_free(&devlist);devlist=0;
			ftdi_free(ftdi);ftdi=0;
			ftdi_free(ftdi2);ftdi2=0;
			return false;
		}
		//set 10s timeout
		ftdi->usb_read_timeout = FTDI_TIMEOUT_MS;
		// Purge both Rx and Tx buffers 
	    if(status=ftdi_usb_purge_rx_buffer(ftdi2)<0){
			fprintf(stderr, "ftdi_usb_purge_rx_buffer failed:%d\n",status);
			ftdi_list_free(&devlist);devlist=0;
			ftdi_free(ftdi);ftdi=0;
			ftdi_free(ftdi2);ftdi2=0;
			return false;
		}       
	}
    readPos=BUF_USABLE_SIZE;//buf is empty
	return true;
}
//return the number of bytes actually received, it should be equal to len, otherwise this indicates an error.
//if it is less than len, subsequent calls are likely to fail.
unsigned int kidekin_trng_read(uint8_t *rxBuffer, unsigned int len){
	if(readPos==-1) 
		return -1;
	unsigned int out=0;
	while(len){
		if(readPos==BUF_USABLE_SIZE){
			readPos=-1;
			int cnt;
			int status=ftdi_read_data(ftdi2,buf,BUF_USABLE_SIZE);
			if(status<=0) {
				printf("ftdi_read_data=%d\n",status); 
				return -1;
			}
			cnt=status;
			if(cnt!=BUF_USABLE_SIZE){ 
				printf("ftdi_read_data=%d\n",cnt); 
				return -1;
			}
			readPos = 0;
		} 
		unsigned int remaining = BUF_USABLE_SIZE - readPos;
		unsigned int toRead = remaining;
		if(toRead > len) toRead = len;
		memcpy(rxBuffer+out,buf+readPos,toRead);
		readPos +=toRead;
		len-=toRead;
		out+=toRead;
	}
	return out;
}
void kidekin_trng_close(void){
	if(verbose) printf("kidekin_trng_close() starts\n");
	if(ftdi2!=0) ftdi_free(ftdi2);ftdi2=0;
	if(devlist!=0) ftdi_list_free(&devlist);devlist=0;
	if(ftdi!=0) ftdi_free(ftdi);ftdi=0;
	if(verbose) printf("kidekin_trng_close() done\n");
}



